@@ -7,7 +7,6 @@ API crystallize from actual use in true XP fashion.
 license: GPL v2 or later
 """
 #============================================================
-__version__ = "$Revision: 1.106 $"
 __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>, I.Haywood <ihaywood@gnu.org>"
 
 # stdlib
@@ -27,7 +26,6 @@ from Gnumed.pycommon import gmTools
 
 
 _log = logging.getLogger('gm.business')
-_log.info(__version__)
 
 try:
 	_
@@ -346,8 +344,10 @@ class cAddress(gmBusinessDBObject.cBusinessDBObject):
 		'lat_lon_address'
 	]
 	#--------------------------------------------------------
-	def format(self):
-		return format_address(address = self)
+	def format(self, single_line=False, verbose=False, show_type=False):
+		if single_line:
+			return format_address_single_line(address = self, show_type = False, verbose = verbose)
+		return format_address(address = self, show_type = False)
 #------------------------------------------------------------
 def address_exists(country=None, state=None, urb=None, postcode=None, street=None, number=None, subunit=None):
 
@@ -387,7 +387,7 @@ def create_address(country=None, state=None, urb=None, suburb=None, postcode=Non
 		subunit = subunit
 	)
 	if pk_address is not None:
-		return cAddress(aPK_obj=pk_address)
+		return cAddress(aPK_obj = pk_address)
 
 	cmd = u"""
 SELECT dem.create_address (
@@ -416,7 +416,7 @@ SELECT dem.create_address (
 	if suburb is not None:
 		queries = [{
 			# CAVE: suburb will be ignored if there already is one
-			'cmd': u"update dem.street set suburb = %(suburb)s where id=%(pk_street)s and suburb is Null",
+			'cmd': u"UPDATE dem.street SET suburb = %(suburb)s WHERE id = %(pk_street)s AND suburb IS NULL",
 			'args': {'suburb': suburb, 'pk_street': adr['pk_street']}
 		}]
 		rows, idx = gmPG2.run_rw_queries(queries = queries)
@@ -440,7 +440,7 @@ def delete_address(pk_address=None):
 	rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': {'pk': pk_address}}])
 	return True
 #------------------------------------------------------------
-def format_address(address=None):
+def format_address_single_line(address=None, verbose=False, show_type=False):
 	data = {
 		'pk_adr': address['pk_address'],
 		'street': address['street'],
@@ -456,14 +456,58 @@ def format_address(address=None):
 		'l10n_country': address['l10n_country'],
 		'code_country': address['code_country']
 	}
-	txt = _(
-		'Address [#%(pk_adr)s]\n'
-		' Street: %(street)s%(notes_street)s\n'
-		' Number/Unit: %(number)s%(subunit)s%(notes_subunit)s\n'
-		' Location: %(zip)s %(urb)s%(suburb)s\n'
-		' Region: %(l10n_state)s, %(code_state)s\n'
-		' Country: %(l10n_country)s, %(code_country)s'
-	) % data
+	if show_type:
+		data['type'] = address['l10n_address_type']
+
+	if verbose:
+		if show_type:
+			template = _('%(type)s: %(street)s %(number)s%(subunit)s, %(zip)s %(urb)s %(suburb)s, %(code_state)s, %(code_country)s')
+		else:
+			template = _('%(street)s %(number)s%(subunit)s, %(zip)s %(urb)s %(suburb)s, %(code_state)s, %(code_country)s')
+	else:
+		if show_type:
+			template = _('%(type)s: %(street)s %(number)s%(subunit)s, %(zip)s %(urb)s, %(code_state)s, %(code_country)s')
+		else:
+			template = _('%(street)s %(number)s%(subunit)s, %(zip)s %(urb)s, %(code_state)s, %(code_country)s')
+
+	return template % data
+#------------------------------------------------------------
+def format_address(address=None, show_type=False):
+	data = {
+		'pk_adr': address['pk_address'],
+		'street': address['street'],
+		'notes_street': gmTools.coalesce(address['notes_street'], u'', u' (%s)'),
+		'number': address['number'],
+		'subunit': gmTools.coalesce(address['subunit'], u'', u'/%s'),
+		'notes_subunit': gmTools.coalesce(address['notes_subunit'], u'', u' (%s)'),
+		'zip': address['postcode'],
+		'urb': address['urb'],
+		'suburb': gmTools.coalesce(address['suburb'], u'', u' (%s)'),
+		'l10n_state': address['l10n_state'],
+		'code_state': address['code_state'],
+		'l10n_country': address['l10n_country'],
+		'code_country': address['code_country']
+	}
+	if show_type:
+		data['type'] = address['l10n_address_type']
+		template = _(
+			'Address (%(type)s) [#%(pk_adr)s]\n'
+			' Street: %(street)s%(notes_street)s\n'
+			' Number/Unit: %(number)s%(subunit)s%(notes_subunit)s\n'
+			' Location: %(zip)s %(urb)s%(suburb)s\n'
+			' Region: %(l10n_state)s, %(code_state)s\n'
+			' Country: %(l10n_country)s, %(code_country)s'
+		)
+	else:
+		template = _(
+			'Address [#%(pk_adr)s]\n'
+			' Street: %(street)s%(notes_street)s\n'
+			' Number/Unit: %(number)s%(subunit)s%(notes_subunit)s\n'
+			' Location: %(zip)s %(urb)s%(suburb)s\n'
+			' Region: %(l10n_state)s, %(code_state)s\n'
+			' Country: %(l10n_country)s, %(code_country)s'
+		)
+	txt = template % data
 	return txt.split('\n')
 #------------------------------------------------------------
 def get_address_types(identity=None):
@@ -481,8 +525,39 @@ def get_addresses(order_by=None):
 	cmd = u"SELECT * FROM dem.v_address %s" % order_by
 	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd}], get_col_idx = True)
 	return [ cAddress(row = {'data': r, 'idx': idx, 'pk_field': u'pk_address'}) for r in rows ]
+#------------------------------------------------------------
+def get_address_from_patient_address_pk(pk_patient_address=None):
+	cmd = u"""
+		SELECT * FROM dem.v_address WHERE
+			pk_address = (
+				SELECT id_address
+				FROM dem.lnk_person_org_address
+				WHERE id = %(pk_pat_adr)s
+			)
+	"""
+	args = {'pk_pat_adr': pk_patient_address}
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+	if len(rows) == 0:
+		return None
+	return cAddress(row = {'data': rows[0], 'idx': idx, 'pk_field': u'pk_address'})
 
 #===================================================================
+def get_patient_address(pk_patient_address=None):
+	cmd = u'SELECT * FROM dem.v_pat_addresses WHERE pk_lnk_person_org_address = %(pk)s'
+	args = {'pk': pk_patient_address}
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+	if len(rows) == 0:
+		return None
+	return cPatientAddress(row = {'data': rows[0], 'idx': idx, 'pk_field': u'pk_address'})
+#-------------------------------------------------------------------
+def get_patient_address_by_type(pk_patient=None, adr_type=None):
+	cmd = u'SELECT * FROM dem.v_pat_addresses WHERE pk_identity = %(pat)s AND address_type = %(typ)s'
+	args = {'pat': pk_patient, 'typ': adr_type}
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+	if len(rows) == 0:
+		return None
+	return cPatientAddress(row = {'data': rows[0], 'idx': idx, 'pk_field': u'pk_address'})
+#-------------------------------------------------------------------
 class cPatientAddress(gmBusinessDBObject.cBusinessDBObject):
 
 	_cmd_fetch_payload = u"SELECT * FROM dem.v_pat_addresses WHERE pk_address = %s"
@@ -502,9 +577,10 @@ class cPatientAddress(gmBusinessDBObject.cBusinessDBObject):
 	def get_identities(self, same_lastname=False):
 		pass
 	#--------------------------------------------------------
-	def format(self):
-		txt = format_address(address = self)
-		txt.append(_(' Type: %s') % self._payload[self._idx['l10n_address_type']])
+	def format(self, single_line=False, verbose=False, show_type=True):
+		if single_line:
+			return format_address_single_line(address = self, verbose = verbose, show_type = show_type)
+		txt = format_address(address = self, show_type = show_type)
 		return txt
 #===================================================================
 # communication channels API
@@ -514,10 +590,11 @@ class cCommChannel(gmBusinessDBObject.cBusinessDBObject):
 	_cmd_fetch_payload = u"SELECT * FROM dem.v_person_comms WHERE pk_lnk_identity2comm = %s"
 	_cmds_store_payload = [
 		u"""UPDATE dem.lnk_identity2comm SET
-				fk_address = %(pk_address)s,
+				--fk_address = %(pk_address)s,
 				fk_type = dem.create_comm_type(%(comm_type)s),
 				url = %(url)s,
-				is_confidential = %(is_confidential)s
+				is_confidential = %(is_confidential)s,
+				comment = gm.nullify_empty_string(%(comment)s)
 			WHERE
 				pk = %(pk_lnk_identity2comm)s
 					AND
@@ -527,10 +604,11 @@ class cCommChannel(gmBusinessDBObject.cBusinessDBObject):
 		"""
 	]
 	_updatable_fields = [
-		'pk_address',
+		#'pk_address',
 		'url',
 		'comm_type',
-		'is_confidential'
+		'is_confidential',
+		'comment'
 	]
 #-------------------------------------------------------------------
 class cOrgCommChannel(gmBusinessDBObject.cBusinessDBObject):
@@ -552,8 +630,19 @@ class cOrgCommChannel(gmBusinessDBObject.cBusinessDBObject):
 	_updatable_fields = [
 		'url',
 		'comm_type',
-		'is_confidential'
+		'is_confidential',
+		'comment'
 	]
+	#--------------------------------------------------------
+	def __setitem__(self, attribute, value):
+		# fake it
+		if attribute == u'comment':
+			return
+		gmBusinessDBObject.cBusinessDBObject.__setitem__(self, attribute, value)
+	#--------------------------------------------------------
+	def get_comment(self):
+		return None
+
 #-------------------------------------------------------------------
 def create_comm_channel(comm_medium=None, url=None, is_confidential=False, pk_channel_type=None, pk_identity=None, pk_org_unit=None):
 	"""Create a communications channel for a patient."""
@@ -988,17 +1077,20 @@ if __name__ == "__main__":
 	def test_tag_images():
 		tag = cTagImage(aPK_obj = 1)
 		print tag
-		print get_tag_images()
+		#print get_tag_images()
+	#--------------------------------------------------------
+	def test_get_billing_address():
+		print get_patient_address_by_type(pk_patient = 12, adr_type = u'billing')
 	#--------------------------------------------------------
-
 	#gmPG2.get_connection()
 
 	#test_address_exists()
-	test_create_address()
+	#test_create_address()
 	#test_get_countries()
 	#test_get_country_for_region()
 	#test_delete_tag()
-	#test_tag_images()
+	test_tag_images()
+	#test_get_billing_address()
 
 	sys.exit()
 #============================================================
