@@ -50,7 +50,6 @@ care of all the pre- and post-GUI runtime environment setup.
  Show this help.
 """
 #==========================================================
-__version__ = "$Revision: 1.169 $"
 __author__  = "H. Herb <hherb@gnumed.net>, K. Hilbert <Karsten.Hilbert@gmx.net>, I. Haywood <i.haywood@ugrad.unimelb.edu.au>"
 __license__ = "GPL v2 or later (details at http://www.gnu.org)"
 
@@ -62,6 +61,7 @@ import logging
 import signal
 import os.path
 import shutil
+import stat
 
 
 # do not run as module
@@ -85,8 +85,8 @@ against. Please run GNUmed as a non-root user.
 	sys.exit(1)
 
 #----------------------------------------------------------
-current_client_version = u'1.1.16'
-current_client_branch = u'1.1'
+current_client_version = u'1.3.9'
+current_client_branch = u'1.3'
 
 _log = None
 _cfg = None
@@ -171,13 +171,14 @@ def setup_python_path():
 	if not u'--local-import' in sys.argv:
 		return
 
-	print "GNUmed startup: Running from local source tree."
-	print "-----------------------------------------------"
+	print "Running from local source tree ..."
 
 	local_python_base_dir = os.path.dirname (
 		os.path.abspath(os.path.join(sys.argv[0], '..'))
 	)
 
+	print ' (%s)' % local_python_base_dir
+
 	# does the path exist at all, physically ?
 	# (*broken* links are reported as False)
 	link_name = os.path.join(local_python_base_dir, 'Gnumed')
@@ -191,6 +192,62 @@ def setup_python_path():
 	print "Adjusting PYTHONPATH ..."
 	sys.path.insert(0, local_python_base_dir)
 #==========================================================
+def setup_local_repo_path():
+
+	local_repo_path = os.path.expanduser(os.path.join (
+		'~',
+		'.gnumed',
+		'local_code',
+		str(current_client_branch)
+	))
+	local_wxGladeWidgets_path = os.path.join(local_repo_path, 'Gnumed', 'wxGladeWidgets')
+
+	if not os.path.exists(local_wxGladeWidgets_path):
+		_log.debug('[%s] not found', local_wxGladeWidgets_path)
+		_log.info('local wxGlade widgets repository not available')
+		return
+
+	_log.info('local wxGlade widgets repository found:')
+	_log.info(local_wxGladeWidgets_path)
+
+	if not os.access(local_wxGladeWidgets_path, os.R_OK):
+		_log.error('invalid repo: no read access')
+		return
+
+	all_entries = os.listdir(os.path.join(local_repo_path, 'Gnumed'))
+	_log.debug('repo base contains: %s', all_entries)
+	all_entries.remove('wxGladeWidgets')
+	try:
+		all_entries.remove('__init__.py')
+	except ValueError:
+		_log.error('invalid repo: lacking __init__.py')
+		return
+	try:
+		all_entries.remove('__init__.pyc')
+	except ValueError:
+		pass
+
+	if len(all_entries) > 0:
+		_log.error('insecure repo: additional files or directories found')
+		return
+
+	# repo must be 0700 (rwx------)
+	stat_val = os.stat(local_wxGladeWidgets_path)
+	_log.debug('repo stat(): %s', stat_val)
+	perms = stat.S_IMODE(stat_val.st_mode)
+	_log.debug('repo permissions: %s (octal: %s)', perms, oct(perms))
+	if perms != 448:				# octal 0700
+		if os.name in ['nt']:
+			_log.warning('this platform does not support os.stat() permission checking')
+		else:
+			_log.error('insecure repo: permissions not 0600')
+			return
+
+	print "Activating local wxGlade widgets repository ..."
+	sys.path.insert(0, local_repo_path)
+	_log.debug('sys.path with repo:')
+	_log.debug(sys.path)
+#==========================================================
 def setup_logging():
 	try:
 		from Gnumed.pycommon import gmLog2 as _gmLog2
@@ -204,15 +261,18 @@ def setup_logging():
 	_log = logging.getLogger('gm.launcher')
 #==========================================================
 def log_startup_info():
-	_log.info('Starting up as main module (%s).', __version__)
-	_log.info('GNUmed client version [%s] on branch [%s]', current_client_version, current_client_branch)
-	_log.info('Platform: %s', platform.uname())
-	_log.info('Python %s on %s (%s)', sys.version, sys.platform, os.name)
+	_log.info(u'Starting up as main module.')
+	_log.info(u'GNUmed client version [%s] on branch [%s]', current_client_version, current_client_branch)
+	_log.info(u'Platform: %s', platform.uname())
+	_log.info(u'Python %s on %s (%s)', sys.version, sys.platform, os.name)
 	try:
 		import lsb_release
-		_log.info('%s' % lsb_release.get_distro_information())
+		_log.info(u'%s' % lsb_release.get_distro_information())
 	except ImportError:
 		pass
+	_log.info('process environment:')
+	for key, val in os.environ.items():
+		_log.info(u' %s: %s', (u'${%s}' % key).rjust(30), val)
 #==========================================================
 def setup_console_exception_handler():
 	from Gnumed.pycommon.gmTools import handle_uncaught_exception_console
@@ -351,7 +411,6 @@ def setup_paths_and_files():
 
 	gmTools.mkdir(os.path.expanduser(os.path.join('~', '.gnumed', 'scripts')))
 	gmTools.mkdir(os.path.expanduser(os.path.join('~', '.gnumed', 'spellcheck')))
-	gmTools.mkdir(os.path.expanduser(os.path.join('~', '.gnumed', 'tmp')))
 	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'export', 'docs')))
 	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'export', 'xDT')))
 	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'export', 'EMR')))
@@ -520,6 +579,7 @@ log_startup_info()
 setup_console_exception_handler()
 setup_cli()
 setup_signal_handlers()
+setup_local_repo_path()
 
 from Gnumed.pycommon import gmI18N, gmTools, gmDateTime, gmHooks
 setup_locale()
