@@ -11,23 +11,32 @@ def resultset_functional_batchgenerator(cursor, size=100):
 			yield rec
 """
 # =======================================================================
-__version__ = "$Revision: 1.127 $"
 __author__  = "K.Hilbert <Karsten.Hilbert@gmx.net>"
-__license__ = 'GPL (details at http://www.gnu.org)'
+__license__ = 'GPL v2 or later (details at http://www.gnu.org)'
 
-### imports ###
 # stdlib
-import time, locale, sys, re as regex, os, codecs, types, datetime as pydt, logging
+import time
+import sys
+import os
+import codecs
+import types
+import logging
+import datetime as pydt
+import re as regex
 
 
 # GNUmed
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmLoginInfo, gmExceptions, gmDateTime, gmBorg, gmI18N, gmLog2
-from Gnumed.pycommon.gmTools import prompted_input
+from Gnumed.pycommon import gmLoginInfo
+from Gnumed.pycommon import gmExceptions
+from Gnumed.pycommon import gmDateTime
+from Gnumed.pycommon import gmBorg
+from Gnumed.pycommon import gmI18N
+from Gnumed.pycommon import gmLog2
+from Gnumed.pycommon.gmTools import prompted_input, u_replacement_character
 
 _log = logging.getLogger('gm.db')
-_log.info(__version__)
 
 
 # 3rd party
@@ -37,7 +46,6 @@ except ImportError:
 	_log.exception("Python database adapter psycopg2 not found.")
 	print "CRITICAL ERROR: Cannot find module psycopg2 for connecting to the database server."
 	raise
-### imports ###
 
 
 _log.info('psycopg2 version: %s' % dbapi.__version__)
@@ -90,49 +98,59 @@ auto_request_login_params = True
 # =======================================================================
 
 known_schema_hashes = {
-	'devel': 'not released, testing only',
-	'v2': 'b09d50d7ed3f91ddf4c4ddb8ea507720',
-	'v3': 'e73718eaf230d8f1d2d01afa8462e176',
-	'v4': '4428ccf2e54c289136819e701bb095ea',
-	'v5': '7e7b093af57aea48c288e76632a382e5',	# ... old (v1) style hashes
-	'v6': '90e2026ac2efd236da9c8608b8685b2d',	# new (v2) style hashes ...
-	'v7': '6c9f6d3981483f8e9433df99d1947b27',
-	'v8': '89b13a7af83337c3aad153b717e52360',
-	'v9': '641a9b2be3c378ffc2bb2f0b1c9f051d',
-	'v10': '7ef42a8fb2bd929a2cdd0c63864b4e8a',
-	'v11': '03042ae24f3f92877d986fb0a6184d76',
-	'v12': '06183a6616db62257e22814007a8ed07',
-	'v13': 'fab7c1ae408a6530c47f9b5111a0841e',
-	'v14': 'e170d543f067d1ea60bfe9076b1560cf',
-	'v15': '70012ff960b77ecdff4981c94b5b55b6'
+	0: 'not released, testing only',
+	2: 'b09d50d7ed3f91ddf4c4ddb8ea507720',
+	3: 'e73718eaf230d8f1d2d01afa8462e176',
+	4: '4428ccf2e54c289136819e701bb095ea',
+	5: '7e7b093af57aea48c288e76632a382e5',	# ... old (v1) style hashes
+	6: '90e2026ac2efd236da9c8608b8685b2d',	# new (v2) style hashes ...
+	7: '6c9f6d3981483f8e9433df99d1947b27',
+	8: '89b13a7af83337c3aad153b717e52360',
+	9: '641a9b2be3c378ffc2bb2f0b1c9f051d',
+	10: '7ef42a8fb2bd929a2cdd0c63864b4e8a',
+	11: '03042ae24f3f92877d986fb0a6184d76',
+	12: '06183a6616db62257e22814007a8ed07',
+	13: 'fab7c1ae408a6530c47f9b5111a0841e',
+	14: 'e170d543f067d1ea60bfe9076b1560cf',
+	15: '70012ff960b77ecdff4981c94b5b55b6',
+	16: '0bcf44ca22c479b52976e5eda1de8161',
+	17: '161428ee97a00e3bf56168c3a15b7b50',
+	18: 'a0f9efcabdecfb4ddb6d8c0b69c02092'
 }
 
 map_schema_hash2version = {
-	'b09d50d7ed3f91ddf4c4ddb8ea507720': 'v2',
-	'e73718eaf230d8f1d2d01afa8462e176': 'v3',
-	'4428ccf2e54c289136819e701bb095ea': 'v4',
-	'7e7b093af57aea48c288e76632a382e5': 'v5',
-	'90e2026ac2efd236da9c8608b8685b2d': 'v6',
-	'6c9f6d3981483f8e9433df99d1947b27': 'v7',
-	'89b13a7af83337c3aad153b717e52360': 'v8',
-	'641a9b2be3c378ffc2bb2f0b1c9f051d': 'v9',
-	'7ef42a8fb2bd929a2cdd0c63864b4e8a': 'v10',
-	'03042ae24f3f92877d986fb0a6184d76': 'v11',
-	'06183a6616db62257e22814007a8ed07': 'v12',
-	'fab7c1ae408a6530c47f9b5111a0841e': 'v13',
-	'e170d543f067d1ea60bfe9076b1560cf': 'v14',
-	'70012ff960b77ecdff4981c94b5b55b6': 'v15'
+	'b09d50d7ed3f91ddf4c4ddb8ea507720': 2,
+	'e73718eaf230d8f1d2d01afa8462e176': 3,
+	'4428ccf2e54c289136819e701bb095ea': 4,
+	'7e7b093af57aea48c288e76632a382e5': 5,
+	'90e2026ac2efd236da9c8608b8685b2d': 6,
+	'6c9f6d3981483f8e9433df99d1947b27': 7,
+	'89b13a7af83337c3aad153b717e52360': 8,
+	'641a9b2be3c378ffc2bb2f0b1c9f051d': 9,
+	'7ef42a8fb2bd929a2cdd0c63864b4e8a': 10,
+	'03042ae24f3f92877d986fb0a6184d76': 11,
+	'06183a6616db62257e22814007a8ed07': 12,
+	'fab7c1ae408a6530c47f9b5111a0841e': 13,
+	'e170d543f067d1ea60bfe9076b1560cf': 14,
+	'70012ff960b77ecdff4981c94b5b55b6': 15,
+	'0bcf44ca22c479b52976e5eda1de8161': 16,
+	'161428ee97a00e3bf56168c3a15b7b50': 17,
+	'a0f9efcabdecfb4ddb6d8c0b69c02092': 18
 }
 
 map_client_branch2required_db_version = {
-	u'GIT tree': u'devel',
-	u'0.3': u'v9',
-	u'0.4': u'v10',
-	u'0.5': u'v11',
-	u'0.6': u'v12',
-	u'0.7': u'v13',
-	u'0.8': u'v14',
-	u'0.9': u'v15'
+	u'GIT tree': 0,
+	u'0.3': 9,
+	u'0.4': 10,
+	u'0.5': 11,
+	u'0.6': 12,
+	u'0.7': 13,
+	u'0.8': 14,
+	u'0.9': 15,
+	u'1.0': 16,		# intentional duplicate with 1.1
+	u'1.1': 16,
+	u'1.2': 17,
+	u'1.3': 18
 }
 
 # get columns and data types for a given table
@@ -159,6 +177,28 @@ where
 order by
 	cols.ordinal_position"""
 
+# only works for single-column FKs but that's fine
+# needs gm-dbo, any-doc won't work
+SQL_foreign_key_name = u"""SELECT tc.constraint_schema, tc.constraint_name
+FROM
+	information_schema.table_constraints tc
+		INNER JOIN information_schema.constraint_column_usage ccu USING (constraint_catalog, constraint_schema, constraint_name)
+			INNER JOIN information_schema.key_column_usage kcu USING (constraint_catalog, constraint_schema, constraint_name)
+WHERE
+	tc.constraint_type = 'FOREIGN KEY'
+		AND
+	kcu.table_schema = %(src_schema)s
+		AND
+	kcu.table_name = %(src_tbl)s
+		AND
+	kcu.column_name = %(src_col)s
+		AND
+	ccu.table_schema = %(target_schema)s
+		AND
+	ccu.table_name = %(target_tbl)s
+		AND
+	ccu.column_name = %(target_col)s"""
+
 # =======================================================================
 # module globals API
 # =======================================================================
@@ -315,7 +355,7 @@ def __request_login_params_tui():
 	print "\nPlease enter the required login parameters:"
 	try:
 		login.host = prompted_input(prompt = "host ('' = non-TCP/IP)", default = '')
-		login.database = prompted_input(prompt = "database", default = 'gnumed_v15')
+		login.database = prompted_input(prompt = "database", default = 'gnumed_v18')
 		login.user = prompted_input(prompt = "user name", default = '')
 		tmp = 'password for "%s" (not shown): ' % login.user
 		login.password = getpass.getpass(tmp)
@@ -425,8 +465,15 @@ def set_default_login(login=None):
 	dsn = make_psycopg2_dsn(login.database, login.host, login.port, login.user, login.password)
 
 	global _default_dsn
+	if _default_dsn is None:
+		old_dsn = u'None'
+	else:
+		old_dsn = regex.sub(r'password=[^\s]+', u'password=%s' % u_replacement_character, _default_dsn)
+	_log.info ('setting default DSN from [%s] to [%s]',
+		old_dsn,
+		regex.sub(r'password=[^\s]+', u'password=%s' % u_replacement_character, dsn)
+	)
 	_default_dsn = dsn
-	_log.info('setting default DSN from [%s] to [%s]' % (_default_dsn, dsn))
 
 	return True
 # =======================================================================
@@ -434,10 +481,10 @@ def set_default_login(login=None):
 # =======================================================================
 def database_schema_compatible(link_obj=None, version=None, verbose=True):
 	expected_hash = known_schema_hashes[version]
-	if version == 'devel':
-		args = {'ver': '9999'}
+	if version == 0:
+		args = {'ver': 9999}
 	else:
-		args = {'ver': version.strip('v')}
+		args = {'ver': version}
 	rows, idx = run_ro_queries (
 		link_obj = link_obj,
 		queries = [{
@@ -451,10 +498,10 @@ def database_schema_compatible(link_obj=None, version=None, verbose=True):
 		_log.error('detected: %s (%s)' % (get_schema_version(link_obj=link_obj), rows[0]['md5']))
 		if verbose:
 			_log.debug('schema dump follows:')
-			for line in get_schema_structure(link_obj=link_obj).split():
+			for line in get_schema_structure(link_obj = link_obj).split():
 				_log.debug(line)
 			_log.debug('schema revision history dump follows:')
-			for line in get_schema_revision_history(link_obj=link_obj):
+			for line in get_schema_revision_history(link_obj = link_obj):
 				_log.debug(u' - '.join(line))
 		return False
 	_log.info('detected schema version [%s], hash [%s]' % (map_schema_hash2version[rows[0]['md5']], rows[0]['md5']))
@@ -471,16 +518,32 @@ def get_schema_structure(link_obj=None):
 	rows, idx = run_ro_queries(link_obj=link_obj, queries = [{'cmd': u'select gm.concat_table_structure()'}])
 	return rows[0][0]
 #------------------------------------------------------------------------
+def get_schema_hash(link_obj=None):
+	rows, idx = run_ro_queries(link_obj=link_obj, queries = [{'cmd': u'select md5(gm.concat_table_structure()) as md5'}])
+	return rows[0]['md5']
+#------------------------------------------------------------------------
 def get_schema_revision_history(link_obj=None):
-	cmd = u"""
-select
-	imported::text,
-	version,
-	filename
-from gm.schema_revision
-order by imported
-"""
-	rows, idx = run_ro_queries(link_obj=link_obj, queries = [{'cmd': cmd}])
+
+	if table_exists(link_obj = link_obj, schema = 'gm', table = 'schema_revision'):
+		cmd = u"""
+			SELECT
+				imported::text,
+				version,
+				filename
+			FROM gm.schema_revision
+			ORDER BY imported"""
+	elif table_exists(link_obj = link_obj, schema = 'public', table = 'gm_schema_revision'):
+		cmd = u"""
+			SELECT
+				imported::text,
+				version,
+				filename
+			FROM public.gm_schema_revision
+			ORDER BY imported"""
+	else:
+		return []
+
+	rows, idx = run_ro_queries(link_obj = link_obj, queries = [{'cmd': cmd}])
 	return rows
 #------------------------------------------------------------------------
 def get_current_user():
@@ -538,6 +601,27 @@ where
 	)
 
 	return rows
+
+#------------------------------------------------------------------------
+def get_foreign_key_names(src_schema=None, src_table=None, src_column=None, target_schema=None, target_table=None, target_column=None, link_obj=None):
+
+	args = {
+		'src_schema': src_schema,
+		'src_tbl': src_table,
+		'src_col': src_column,
+		'target_schema': target_schema,
+		'target_tbl': target_table,
+		'target_col': target_column
+	}
+
+	rows, idx = run_ro_queries (
+		link_obj = link_obj,
+		queries = [{'cmd': SQL_foreign_key_name, 'args': args}],
+		get_col_idx = False
+	)
+
+	return rows
+
 #------------------------------------------------------------------------
 def get_child_tables(schema='public', table=None, link_obj=None):
 	"""Return child tables of <table>."""
@@ -561,6 +645,12 @@ where
 	rows, idx = run_ro_queries(link_obj = link_obj, queries = [{'cmd': cmd, 'args': {'schema': schema, 'table': table}}])
 	return rows
 #------------------------------------------------------------------------
+def schema_exists(link_obj=None, schema=u'gm'):
+	cmd = u"""SELECT EXISTS (SELECT 1 FROM pg_namespace WHERE nspname = %(schema)s)"""
+	args = {'schema': schema}
+	rows, idx = run_ro_queries(link_obj = link_obj, queries = [{'cmd': cmd, 'args': args}])
+	return rows[0][0]
+#------------------------------------------------------------------------
 def table_exists(link_obj=None, schema=None, table=None):
 	"""Returns false, true."""
 	cmd = u"""
@@ -623,16 +713,17 @@ def export_translations_from_database(filename=None):
 	tx_file = codecs.open(filename, 'wb', 'utf8')
 	tx_file.write(u'-- GNUmed database string translations exported %s\n' % gmDateTime.pydt_now_here().strftime('%Y-%m-%d %H:%M'))
 	tx_file.write(u'-- - contains translations for each of [%s]\n' % u', '.join(get_translation_languages()))
-	tx_file.write(u'-- - user language is set to [%s]\n\n' % get_current_user_language())
+	tx_file.write(u'-- - user database language is set to [%s]\n\n' % get_current_user_language())
 	tx_file.write(u'-- Please email this file to <gnumed-devel@gnu.org>.\n')
 	tx_file.write(u'-- ----------------------------------------------------------------------------------------------\n\n')
-	tx_file.write(u'set default_transaction_read_only to off\n\n')
+	tx_file.write(u'set default_transaction_read_only to off;\n\n')
+	tx_file.write(u"set client_encoding to 'utf-8';\n\n")
 	tx_file.write(u'\\unset ON_ERROR_STOP\n\n')
 
 	cmd = u'SELECT lang, orig, trans FROM i18n.translations ORDER BY lang, orig'
 	rows, idx = run_ro_queries(queries = [{'cmd': cmd}], get_col_idx = False)
 	for row in rows:
-		line = u"select i18n.upd_tx(quote_literal(E'%s'), quote_literal(E'%s'), quote_literal(E'%s'));\n" % (
+		line = u"select i18n.upd_tx(E'%s', E'%s', E'%s');\n" % (
 			row['lang'].replace("'", "\\'"),
 			row['orig'].replace("'", "\\'"),
 			row['trans'].replace("'", "\\'")
@@ -789,109 +880,7 @@ def force_user_language(language=None):
 		'cmd': u'select i18n.force_curr_lang(%(lang)s)',
 		'args': {'lang': language}
 	}])
-#------------------------------------------------------------------------
-#------------------------------------------------------------------------
-text_expansion_keywords = None
 
-def get_text_expansion_keywords():
-	global text_expansion_keywords
-	if text_expansion_keywords is not None:
-		return text_expansion_keywords
-
-	cmd = u"""select keyword, public_expansion, private_expansion, owner from clin.v_keyword_expansions"""
-	rows, idx = run_ro_queries(queries = [{'cmd': cmd}])
-	text_expansion_keywords = rows
-
-	_log.info('retrieved %s text expansion keywords', len(text_expansion_keywords))
-
-	return text_expansion_keywords
-#------------------------------------------------------------------------
-def expand_keyword(keyword = None):
-
-	# Easter Egg ;-)
-	if keyword == u'$$steffi':
-		return u'Hai, play !  Versucht das ! (Keks dazu ?)  :-)'
-
-	cmd = u"""select expansion from clin.v_your_keyword_expansions where keyword = %(kwd)s"""
-	rows, idx = run_ro_queries(queries = [{'cmd': cmd, 'args': {'kwd': keyword}}])
-
-	if len(rows) == 0:
-		return None
-
-	return rows[0]['expansion']
-#------------------------------------------------------------------------
-def get_keyword_expansion_candidates(keyword = None):
-
-	if keyword is None:
-		return []
-
-	get_text_expansion_keywords()
-
-	candidates = []
-	for kwd in text_expansion_keywords:
-		if kwd['keyword'].startswith(keyword):
-			candidates.append(kwd['keyword'])
-
-	return candidates
-#------------------------------------------------------------------------
-def add_text_expansion(keyword=None, expansion=None, public=None):
-
-	if public:
-		cmd = u"select 1 from clin.v_keyword_expansions where public_expansion is true and keyword = %(kwd)s"
-	else:
-		cmd = u"select 1 from clin.v_your_keyword_expansions where private_expansion is true and keyword = %(kwd)s"
-
-	rows, idx = run_ro_queries(queries = [{'cmd': cmd, 'args': {'kwd': keyword}}])
-	if len(rows) != 0:
-		return False
-
-	if public:
-		cmd = u"""
-insert into clin.keyword_expansion (keyword, expansion, fk_staff)
-values (%(kwd)s, %(exp)s, null)"""
-	else:
-		cmd = u"""
-insert into clin.keyword_expansion (keyword, expansion, fk_staff)
-values (%(kwd)s, %(exp)s, (select pk from dem.staff where db_user = current_user))"""
-
-	rows, idx = run_rw_queries(queries = [{'cmd': cmd, 'args': {'kwd': keyword, 'exp': expansion}}])
-
-	global text_expansion_keywords
-	text_expansion_keywords = None
-
-	return True
-#------------------------------------------------------------------------
-def delete_text_expansion(keyword):
-	cmd = u"""
-delete from clin.keyword_expansion where
-	keyword = %(kwd)s and (
-		(fk_staff = (select pk from dem.staff where db_user = current_user))
-			or
-		(fk_staff is null and owner = current_user)
-	)"""
-	rows, idx = run_rw_queries(queries = [{'cmd': cmd, 'args': {'kwd': keyword}}])
-
-	global text_expansion_keywords
-	text_expansion_keywords = None
-#------------------------------------------------------------------------
-def edit_text_expansion(keyword, expansion):
-
-	cmd1 = u"""
-delete from clin.keyword_expansion where
-	keyword = %(kwd)s and 
-	fk_staff = (select pk from dem.staff where db_user = current_user)"""
-
-	cmd2 = u"""
-insert into clin.keyword_expansion (keyword, expansion, fk_staff)
-values (%(kwd)s, %(exp)s, (select pk from dem.staff where db_user = current_user))"""
-
-	rows, idx = run_rw_queries(queries = [
-		{'cmd': cmd1, 'args': {'kwd': keyword}},
-		{'cmd': cmd2, 'args': {'kwd': keyword, 'exp': expansion}},
-	])
-
-	global text_expansion_keywords
-	text_expansion_keywords = None
 # =======================================================================
 # query runners and helpers
 # =======================================================================
@@ -904,12 +893,12 @@ def send_maintenance_shutdown():
 	run_rw_queries(queries = [{'cmd': cmd}], return_data = False)
 #------------------------------------------------------------------------
 def is_pg_interval(candidate=None):
-	cmd = u'select %(candidate)s::interval'
+	cmd = u'SELECT %(candidate)s::interval'
 	try:
 		rows, idx = run_ro_queries(queries = [{'cmd': cmd, 'args': {'candidate': candidate}}])
 		return True
 	except:
-		cmd = u'select %(candidate)s::text::interval'
+		cmd = u'SELECT %(candidate)s::text::interval'
 		try:
 			rows, idx = run_ro_queries(queries = [{'cmd': cmd, 'args': {'candidate': candidate}}])
 			return True
@@ -963,20 +952,17 @@ def bytea2file_object(data_query=None, file_obj=None, chunk_size=0, data_size=No
 			conn.rollback()
 			return True
 
-	_log.debug('expecting bytea data of size: [%s] bytes' % data_size)
-	_log.debug('using chunk size of: [%s] bytes' % chunk_size)
-
-	# chunk size of 0 means "retrieve whole field at once"
+	max_chunk_size = 1024 * 1024 * 20			# 20 MB, works for typical CR DICOMs
 	if chunk_size == 0:
-		chunk_size = data_size
-		_log.debug('chunk size [0] bytes: retrieving all data at once')
+		chunk_size = min(data_size, max_chunk_size)
+
+	_log.debug('expecting %s bytes of BYTEA data in chunks of %s bytes', data_size, chunk_size)
 
 	# Windoze sucks: it can't transfer objects of arbitrary size,
 	# anyways, we need to split the transfer,
 	# however, only possible if postgres >= 7.2
 	needed_chunks, remainder = divmod(data_size, chunk_size)
-	_log.debug('chunks to retrieve: [%s]' % needed_chunks)
-	_log.debug('remainder to retrieve: [%s] bytes' % remainder)
+	_log.debug('# of chunks: %s; remainder: %s bytes', needed_chunks, remainder)
 
 	# try setting "bytea_output"
 	# - fails if not necessary
@@ -1121,17 +1107,48 @@ def run_ro_queries(link_obj=None, queries=None, verbose=False, return_data=True,
 			curs.execute(query['cmd'], args)
 			if verbose:
 				_log.debug('ran query: [%s]', curs.query)
-				_log.debug('PG status message: %s', curs.statusmessage)
+				if curs.statusmessage != u'':
+					_log.debug('PG status message: %s', curs.statusmessage)
 				_log.debug('cursor description: %s', str(curs.description))
-		except:
-			# FIXME: use .pgcode
+		except dbapi.Error as pg_exc:
+			_log.error('query failed: [%s]', curs.query)
+			if curs.statusmessage != u'':
+				_log.error('PG status message: %s', curs.statusmessage)
+			_log.error('PG error code: %s', pg_exc.pgcode)
+			if pg_exc.pgerror is not None:
+				_log.error('PG error message: %s', pg_exc.pgerror.strip().strip(u'\n').strip().strip(u'\n'))
 			try:
 				curs_close()
 			except dbapi.InterfaceError:
 				_log.exception('cannot close cursor')
 			tx_rollback()		# need to rollback so ABORT state isn't preserved in pooled conns
+			if pg_exc.pgcode == sql_error_codes.INSUFFICIENT_PRIVILEGE:
+				details = u'Query: [%s]' % curs.query.strip().strip(u'\n').strip().strip(u'\n')
+				if curs.statusmessage != u'':
+					details = u'Status: %s\n%s' % (
+						curs.statusmessage.strip().strip(u'\n').strip().strip(u'\n'),
+						details
+					)
+				if pg_exc.pgerror is None:
+					msg = u'[%s]: %s' % (pg_exc.pgcode, pg_exc.pgerror)
+				else:
+					msg = u'[%s]: %s' % (pg_exc.pgcode, pg_exc.pgerror.strip().strip(u'\n').strip().strip(u'\n'))
+				raise gmExceptions.AccessDenied (
+					msg,
+					source = u'PostgreSQL',
+					code = pg_exc.pgcode,
+					details = details
+				)
+			raise
+		except:
 			_log.error('query failed: [%s]', curs.query)
-			_log.error('PG status message: %s', curs.statusmessage)
+			if curs.statusmessage != u'':
+				_log.error('PG status message: %s', curs.statusmessage)
+			try:
+				curs_close()
+			except dbapi.InterfaceError:
+				_log.exception('cannot close cursor')
+			tx_rollback()		# need to rollback so ABORT state isn't preserved in pooled conns
 			raise
 
 	data = None
@@ -1161,7 +1178,7 @@ def run_rw_queries(link_obj=None, queries=None, end_tx=False, return_data=None,
 		is a list of dicts [{'cmd': <string>, 'args': <dict> or <tuple>)
 		to be executed as a single transaction, the last
 		query may usefully return rows (such as a
-		"select currval('some_sequence')" statement)
+		"SELECT currval('some_sequence')" statement)
 
 	<end_tx>
 		- controls whether the transaction is finalized (eg.
@@ -1197,24 +1214,24 @@ def run_rw_queries(link_obj=None, queries=None, end_tx=False, return_data=None,
 	if isinstance(link_obj, dbapi._psycopg.cursor):
 		conn_close = __noop
 		conn_commit = __noop
-		conn_rollback = __noop
+		tx_rollback = __noop
 		curs = link_obj
 		curs_close = __noop
 	elif isinstance(link_obj, dbapi._psycopg.connection):
 		conn_close = __noop
 		if end_tx:
 			conn_commit = link_obj.commit
-			conn_rollback = link_obj.rollback
+			tx_rollback = link_obj.rollback
 		else:
 			conn_commit = __noop
-			conn_rollback = __noop
+			tx_rollback = __noop
 		curs = link_obj.cursor()
 		curs_close = curs.close
 	elif link_obj is None:
 		conn = get_connection(readonly=False)
 		conn_close = conn.close
 		conn_commit = conn.commit
-		conn_rollback = conn.rollback
+		tx_rollback = conn.rollback
 		curs = conn.cursor()
 		curs_close = curs.close
 	else:
@@ -1230,12 +1247,43 @@ def run_rw_queries(link_obj=None, queries=None, end_tx=False, return_data=None,
 			args = None
 		try:
 			curs.execute(query['cmd'], args)
+		except dbapi.Error as pg_exc:
+			_log.error('RW query failed: [%s]', curs.query)
+			if curs.statusmessage != u'':
+				_log.error('PG status message: %s', curs.statusmessage)
+			_log.error('PG error code: %s', pg_exc.pgcode)
+			if pg_exc.pgerror is not None:
+				_log.error('PG error message: %s', pg_exc.pgerror.strip().strip(u'\n').strip().strip(u'\n'))
+			try:
+				curs_close()
+				tx_rollback()			# just for good measure
+				conn_close()
+			except dbapi.InterfaceError:
+				_log.exception('cannot cleanup')
+			if pg_exc.pgcode == sql_error_codes.INSUFFICIENT_PRIVILEGE:
+				details = u'Query: [%s]' % curs.query.strip().strip(u'\n').strip().strip(u'\n')
+				if curs.statusmessage != u'':
+					details = u'Status: %s\n%s' % (
+						curs.statusmessage.strip().strip(u'\n').strip().strip(u'\n'),
+						details
+					)
+				if pg_exc.pgerror is None:
+					msg = u'[%s]: %s' % (pg_exc.pgcode, pg_exc.pgerror)
+				else:
+					msg = u'[%s]: %s' % (pg_exc.pgcode, pg_exc.pgerror.strip().strip(u'\n').strip().strip(u'\n'))
+				raise gmExceptions.AccessDenied (
+					msg,
+					source = u'PostgreSQL',
+					code = pg_exc.pgcode,
+					details = details
+				)
+			raise
 		except:
 			_log.exception('error running RW query')
 			gmLog2.log_stack_trace()
 			try:
 				curs_close()
-				conn_rollback()
+				tx_rollback()
 				conn_close()
 			except dbapi.InterfaceError:
 				_log.exception('cannot cleanup')
@@ -1252,7 +1300,7 @@ def run_rw_queries(link_obj=None, queries=None, end_tx=False, return_data=None,
 			gmLog2.log_stack_trace()
 			try:
 				curs_close()
-				conn_rollback()
+				tx_rollback()
 				conn_close()
 			except dbapi.InterfaceError:
 				_log.exception('cannot cleanup')
@@ -1385,16 +1433,18 @@ def get_raw_connection(dsn=None, verbose=False, readonly=True):
 	global postgresql_version
 	if postgresql_version is None:
 		curs = conn.cursor()
-		curs.execute ("""
-			select
-				substring(setting, '^\d{1,2}\.\d{1,2}')::numeric AS version
-			from pg_settings
-			where name='server_version'"""
-		)
+		curs.execute("""
+			SELECT
+				substring(setting, E'^\\\\d{1,2}\\\\.\\\\d{1,2}')::numeric AS version
+			FROM
+				pg_settings
+			WHERE
+				name = 'server_version'
+		""")
 		postgresql_version = curs.fetchone()['version']
 		_log.info('PostgreSQL version (numeric): %s' % postgresql_version)
 		try:
-			curs.execute("select pg_size_pretty(pg_database_size(current_database()))")
+			curs.execute("SELECT pg_size_pretty(pg_database_size(current_database()))")
 			_log.info('database size: %s', curs.fetchone()[0])
 		except:
 			pass
@@ -1473,36 +1523,18 @@ def get_connection(dsn=None, readonly=True, encoding=None, verbose=False, pooled
 
 	# - transaction isolation level
 	if readonly:
-		conn.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_READ_COMMITTED)
 		iso_level = u'read committed'
 	else:
 		conn.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_SERIALIZABLE)
 		iso_level = u'serializable'
 
-	_log.debug('client string encoding [%s], isolation level [%s], time zone [%s], datestyle [ISO], sql_inheritance [ON]', encoding, iso_level, _default_client_timezone)
+	_log.debug('client string encoding [%s], isolation level [%s], time zone [%s]', encoding, iso_level, _default_client_timezone)
 
 	curs = conn.cursor()
 
 	# - client time zone
 	curs.execute(_sql_set_timezone, [_default_client_timezone])
 
-	# - datestyle
-	# regarding DMY/YMD handling: since we force *input* to
-	# ISO, too, the DMY/YMD setting is not needed
-	cmd = "set datestyle to 'ISO'"
-	curs.execute(cmd)
-
-	# - SQL inheritance mode
-	cmd = 'set sql_inheritance to on'
-	curs.execute(cmd)
-
-	# - version string
-	global postgresql_version_string
-	if postgresql_version_string is None:
-		curs.execute('select version()')
-		postgresql_version_string = curs.fetchone()['version']
-		_log.info('PostgreSQL version (string): "%s"' % postgresql_version_string)
-
 	conn.commit()
 
 	# FIXME: remove this whole affair once either 9.0 is standard (Ubuntu 10 LTS is
@@ -1558,7 +1590,7 @@ def sanity_check_time_skew(tolerance=60):
 	"""
 	_log.debug('maximum skew tolerance (seconds): %s', tolerance)
 
-	cmd = u"select now() at time zone 'UTC'"
+	cmd = u"SELECT now() at time zone 'UTC'"
 	conn = get_raw_connection(readonly=True)
 	curs = conn.cursor()
 
@@ -1603,16 +1635,30 @@ def sanity_check_database_settings():
 		2: fatal problem
 	"""
 	_log.debug('checking database settings')
-	settings = {
+
+	conn = get_connection()
+
+	# - version string
+	global postgresql_version_string
+	if postgresql_version_string is None:
+		curs = conn.cursor()
+		curs.execute('SELECT version()')
+		postgresql_version_string = curs.fetchone()['version']
+		curs.close()
+		_log.info('PostgreSQL version (string): "%s"' % postgresql_version_string)
+
+	options2check = {
 		# setting: [expected value, risk, fatal?]
 		u'allow_system_table_mods': [u'off', u'system breakage', False],
 		u'check_function_bodies': [u'on', u'suboptimal error detection', False],
+		u'datestyle': [u'ISO', u'faulty timestamp parsing', True],
+		u'default_transaction_isolation': [u'read committed', u'faulty database reads', True],
 		u'default_transaction_read_only': [u'on', u'accidental database writes', False],
 		u'fsync': [u'on', u'data loss/corruption', True],
 		u'full_page_writes': [u'on', u'data loss/corruption', False],
 		u'lc_messages': [u'C', u'suboptimal error detection', False],
 		u'password_encryption': [u'on', u'breach of confidentiality', False],
-		#u'regex_flavor': [u'advanced', u'query breakage', False],					# 9.0 doesn't support this anymore, default now advanced anyway
+		u'regex_flavor': [u'advanced', u'query breakage', False],					# 9.0 doesn't support this anymore, default now advanced anyway
 		u'synchronous_commit': [u'on', u'data loss/corruption', False],
 		u'sql_inheritance': [u'on', u'query breakage, data loss/corruption', True]
 	}
@@ -1620,32 +1666,41 @@ def sanity_check_database_settings():
 	from Gnumed.pycommon import gmCfg2
 	_cfg = gmCfg2.gmCfgData()
 	if _cfg.get(option = u'hipaa'):
-		settings[u'log_connections'] = [u'on', u'non-compliance with HIPAA', True]
-		settings[u'log_disconnections'] = [u'on', u'non-compliance with HIPAA', True]
+		options2check[u'log_connections'] = [u'on', u'non-compliance with HIPAA', True]
+		options2check[u'log_disconnections'] = [u'on', u'non-compliance with HIPAA', True]
 	else:
-		settings[u'log_connections'] = [u'on', u'non-compliance with HIPAA', None]
-		settings[u'log_disconnections'] = [u'on', u'non-compliance with HIPAA', None]
+		options2check[u'log_connections'] = [u'on', u'non-compliance with HIPAA', None]
+		options2check[u'log_disconnections'] = [u'on', u'non-compliance with HIPAA', None]
 
-	cmd = u"select name, setting from pg_settings where name in %(settings)s"
-	rows, idx = run_ro_queries(queries = [{'cmd': cmd, 'args': {'settings': tuple(settings.keys())}}])
+	cmd = u"SELECT name, setting from pg_settings where name in %(settings)s"
+	rows, idx = run_ro_queries (
+		link_obj = conn,
+		queries = [{'cmd': cmd, 'args': {'settings': tuple(options2check.keys())}}],
+		get_col_idx = False
+	)
 
 	found_error = False
 	found_problem = False
 	msg = []
 	for row in rows:
-		if row[1] != settings[row[0]][0]:
-			if settings[row[0]][2] is True:
+		option = row['name']
+		value_found = row['setting']
+		value_expected = options2check[option][0]
+		risk = options2check[option][1]
+		fatal_setting = options2check[option][2]
+		if value_found != value_expected:
+			if fatal_setting is True:
 				found_error = True
-			elif settings[row[0]][2] is False:
+			elif fatal_setting is False:
 				found_problem = True
-			elif settings[row[0]][2] is None:
+			elif fatal_setting is None:
 				pass
 			else:
-				_log.error(settings[row[0]])
+				_log.error(options2check[option])
 				raise ValueError(u'invalid database configuration sanity check')
-			msg.append(_(' option [%s]: %s') % (row[0], row[1]))
-			msg.append(_('  risk: %s') % settings[row[0]][1])
-			_log.warning('PG option [%s] set to [%s], expected [%s], risk: <%s>' % (row[0], row[1], settings[row[0]][0], settings[row[0]][1]))
+			msg.append(_(' option [%s]: %s') % (option, value_found))
+			msg.append(_('  risk: %s') % risk)
+			_log.warning('PG option [%s] set to [%s], expected [%s], risk: <%s>' % (option, value_found, value_expected, risk))
 
 	if found_error:
 		return 2, u'\n'.join(msg)
@@ -1726,26 +1781,26 @@ class cAdapterPyDateTime(object):
 	def getquoted(self):
 		return _timestamp_template % self.__dt.isoformat()
 
-# remove for 0.9
-# ----------------------------------------------------------------------
-#class cAdapterMxDateTime(object):
-#
-#	def __init__(self, dt):
-#		if dt.tz == '???':
-#			_log.info('[%s]: no time zone string available in (%s), assuming local time zone', self.__class__.__name__, dt)
-#		self.__dt = dt
-#
-#	def getquoted(self):
-#		# under some locale settings the mx.DateTime ISO formatter
-#		# will insert "," into the ISO string,
-#		# while this is allowed per the ISO8601 spec PostgreSQL
-#		# cannot currently handle that,
-#		# so map those "," to "." to make things work:
-#		return mxDT.ISO.str(self.__dt).replace(',', '.')
-#
-# ----------------------------------------------------------------------
-# PostgreSQL -> Python
-# ----------------------------------------------------------------------
+## remove for 0.9
+## ----------------------------------------------------------------------
+##class cAdapterMxDateTime(object):
+##
+##	def __init__(self, dt):
+##		if dt.tz == '???':
+##			_log.info('[%s]: no time zone string available in (%s), assuming local time zone', self.__class__.__name__, dt)
+##		self.__dt = dt
+##
+##	def getquoted(self):
+##		# under some locale settings the mx.DateTime ISO formatter
+##		# will insert "," into the ISO string,
+##		# while this is allowed per the ISO8601 spec PostgreSQL
+##		# cannot currently handle that,
+##		# so map those "," to "." to make things work:
+##		return mxDT.ISO.str(self.__dt).replace(',', '.')
+##
+## ----------------------------------------------------------------------
+## PostgreSQL -> Python
+## ----------------------------------------------------------------------
 
 #=======================================================================
 #  main
@@ -1761,15 +1816,15 @@ psycopg2.extensions.register_type(psycopg2._psycopg.UNICODEARRAY)
 # check in 0.9:
 psycopg2.extensions.register_adapter(pydt.datetime, cAdapterPyDateTime)
 
-# remove for 0.9
-try:
-	import mx.DateTime as mxDT
-#	psycopg2.extensions.register_adapter(mxDT.DateTimeType, cAdapterMxDateTime)
-except ImportError:
-	_log.warning('cannot import mx.DateTime')
+## remove for 0.9
+#try:
+#	import mx.DateTime as mxDT
+##	psycopg2.extensions.register_adapter(mxDT.DateTimeType, cAdapterMxDateTime)
+#except ImportError:
+#	_log.warning('cannot import mx.DateTime')
 
 # do NOT adapt *lists* to "... IN (*) ..." syntax because we want
-# them adapted to "... ARRAY()..." so we can support PG arrays
+# them adapted to "... ARRAY[]..." so we can support PG arrays
 
 #=======================================================================
 if __name__ == "__main__":
@@ -1886,20 +1941,20 @@ if __name__ == "__main__":
 		dsn = 'dbname=gnumed_v9 user=any-doc password=any-doc'
 		conn = get_connection(dsn, readonly=True)
 
-		data, idx = run_ro_queries(link_obj=conn, queries=[{'cmd': u'select version()'}], return_data=True, get_col_idx=True, verbose=True)
+		data, idx = run_ro_queries(link_obj=conn, queries=[{'cmd': u'SELECT version()'}], return_data=True, get_col_idx=True, verbose=True)
 		print data
 		print idx
-		data, idx = run_ro_queries(link_obj=conn, queries=[{'cmd': u'select 1'}], return_data=True, get_col_idx=True)
+		data, idx = run_ro_queries(link_obj=conn, queries=[{'cmd': u'SELECT 1'}], return_data=True, get_col_idx=True)
 		print data
 		print idx
 
 		curs = conn.cursor()
 
-		data, idx = run_ro_queries(link_obj=curs, queries=[{'cmd': u'select version()'}], return_data=True, get_col_idx=True, verbose=True)
+		data, idx = run_ro_queries(link_obj=curs, queries=[{'cmd': u'SELECT version()'}], return_data=True, get_col_idx=True, verbose=True)
 		print data
 		print idx
 
-		data, idx = run_ro_queries(link_obj=curs, queries=[{'cmd': u'select 1'}], return_data=True, get_col_idx=True, verbose=True)
+		data, idx = run_ro_queries(link_obj=curs, queries=[{'cmd': u'SELECT 1'}], return_data=True, get_col_idx=True, verbose=True)
 		print data
 		print idx
 
@@ -1996,7 +2051,7 @@ if __name__ == "__main__":
 		dsn = get_default_dsn()
 		conn = get_connection(dsn, readonly=True)
 		curs = conn.cursor()
-		curs.execute('select * from clin.clin_narrative where narrative = %s', ['a'])
+		curs.execute('SELECT * from clin.clin_narrative where narrative = %s', ['a'])
 	#--------------------------------------------------------------------
 	def test_sanitize_pg_regex():
 		tests = [
@@ -2033,14 +2088,6 @@ if __name__ == "__main__":
 	def test_sanity_check_time_skew():
 		sanity_check_time_skew()
 	#--------------------------------------------------------------------
-	def test_keyword_expansion():
-		print "keywords, from database:"
-		print get_text_expansion_keywords()
-		print "keywords, cached:"
-		print get_text_expansion_keywords()
-		print "'$keyword' expands to:"
-		print expand_keyword(keyword = u'$dvt')
-	#--------------------------------------------------------------------
 	def test_get_foreign_key_details():
 		for row in get_foreign_keys2column (
 			schema = u'dem',
@@ -2091,11 +2138,36 @@ if __name__ == "__main__":
 	def test_run_query():
 		gmDateTime.init()
 		args = {'dt': gmDateTime.pydt_max_here()}
-		cmd = u"select %(dt)s"
+		cmd = u"SELECT %(dt)s"
 
-		#cmd = u"select 'infinity'::timestamp with time zone"
+		#cmd = u"SELECT 'infinity'::timestamp with time zone"
+
+		cmd = u"""
+SELECT to_timestamp (foofoo,'YYMMDD.HH24MI') FROM (
+	SELECT REGEXP_REPLACE (
+		't1.130729.0902.tif',			-- string
+		E'(.1)\.([0-9\.]+)(\.tif)',		-- pattern
+		E'\\\\2'						-- replacement
+	) AS foofoo
+) AS foo"""
 		rows, idx = run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
 		print rows
+		print rows[0]
+		print rows[0][0]
+	#--------------------------------------------------------------------
+	def test_schema_exists():
+		print schema_exists()
+	#--------------------------------------------------------------------
+	def test_get_foreign_key_names():
+		print get_foreign_key_names (
+			src_schema = 'dem',
+			src_table = 'names',
+			src_column = 'id_identity',
+			target_schema = 'dem',
+			target_table = 'identity',
+			target_column = 'pk'
+		)
+
 	#--------------------------------------------------------------------
 	# run tests
 	#test_file2bytea()
@@ -2109,10 +2181,11 @@ if __name__ == "__main__":
 	#test_sanitize_pg_regex()
 	#test_is_pg_interval()
 	#test_sanity_check_time_skew()
-	#test_keyword_expansion()
 	#test_get_foreign_key_details()
 	#test_set_user_language()
 	#test_get_schema_revision_history()
-	test_run_query()
+	#test_run_query()
+	#test_schema_exists()
+	test_get_foreign_key_names()
 
 # ======================================================================
