@@ -7,12 +7,14 @@
 	and Karsten <Karsten.Hilbert@gmx.net>.
 """
 #================================================================
-__version__ = "$Revision: 1.114 $"
 __author__ = "cfmoro1976@yahoo.es, karsten.hilbert@gmx.net"
-__license__ = "GPL"
+__license__ = "GPL v2 or later"
 
 # stdlib
-import sys, re, datetime as pydt, logging, time
+import sys
+import time
+import logging
+import datetime as pydt
 
 
 # 3rd party
@@ -22,15 +24,42 @@ import wx
 # GNUmed
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmI18N, gmMatchProvider, gmDispatcher, gmTools, gmDateTime, gmCfg, gmExceptions
-from Gnumed.business import gmEMRStructItems, gmPerson, gmSOAPimporter, gmSurgery, gmPersonSearch
-from Gnumed.wxpython import gmPhraseWheel, gmGuiHelpers, gmListWidgets, gmEditArea, gmPatSearchWidgets
-from Gnumed.wxGladeWidgets import wxgIssueSelectionDlg, wxgMoveNarrativeDlg
-from Gnumed.wxGladeWidgets import wxgEncounterTypeEditAreaPnl
+from Gnumed.pycommon import gmI18N
+from Gnumed.pycommon import gmExceptions
+from Gnumed.pycommon import gmCfg
+from Gnumed.pycommon import gmDateTime
+from Gnumed.pycommon import gmTools
+from Gnumed.pycommon import gmDispatcher
+from Gnumed.pycommon import gmMatchProvider
+
+from Gnumed.business import gmEMRStructItems
+from Gnumed.business import gmSurgery
+from Gnumed.business import gmPerson
+
+from Gnumed.wxpython import gmPhraseWheel
+from Gnumed.wxpython import gmGuiHelpers
+from Gnumed.wxpython import gmListWidgets
+from Gnumed.wxpython import gmEditArea
+from Gnumed.wxpython import gmPatSearchWidgets
 
 
 _log = logging.getLogger('gm.ui')
-_log.info(__version__)
+#================================================================
+# EMR access helper functions
+#----------------------------------------------------------------
+def emr_access_spinner(time2spin=0):
+	"""Spin time in seconds."""
+	if time2spin == 0:
+		return
+	sleep_time = 0.1
+	total_rounds = int(time2spin / sleep_time)
+	if total_rounds < 1:
+		return
+	rounds = 0
+	while rounds < total_rounds:
+		wx.Yield()
+		time.sleep(sleep_time)
+		rounds += 1
 #================================================================
 # performed procedure related widgets/functions
 #----------------------------------------------------------------
@@ -242,11 +271,11 @@ limit 25
 		else:
 			self._DPRW_date.display_as_valid(True)
 
+		start = self._DPRW_date.GetData()
 		end = self._DPRW_end.GetData()
 		self._DPRW_end.display_as_valid(True)
 		if end is not None:
 			end = end.get_pydt()
-			start = self._DPRW_end.GetData()
 			if start is not None:
 				start = start.get_pydt()
 				if end < start:
@@ -431,7 +460,7 @@ limit 25
 			self._DPRW_end.is_valid_timestamp()
 		event.Skip()
 #================================================================
-# hospital stay related widgets/functions
+# hospitalizations related widgets/functions
 #----------------------------------------------------------------
 def manage_hospital_stays(parent=None):
 
@@ -441,6 +470,14 @@ def manage_hospital_stays(parent=None):
 	if parent is None:
 		parent = wx.GetApp().GetTopWindow()
 	#-----------------------------------------
+	def get_tooltip(stay=None):
+		if stay is None:
+			return None
+		return stay.format (
+			include_procedures = True,
+			include_docs = True
+		)
+	#-----------------------------------------
 	def edit(stay=None):
 		return edit_hospital_stay(parent = parent, hospital_stay = stay)
 	#-----------------------------------------
@@ -449,7 +486,7 @@ def manage_hospital_stays(parent=None):
 			return True
 		gmDispatcher.send (
 			signal = u'statustext',
-			msg = _('Cannot delete hospital stay.'),
+			msg = _('Cannot delete hospitalization.'),
 			beep = True
 		)
 		return False
@@ -469,14 +506,15 @@ def manage_hospital_stays(parent=None):
 	#-----------------------------------------
 	gmListWidgets.get_choices_from_list (
 		parent = parent,
-		msg = _('\nSelect the hospital stay you want to edit !\n'),
-		caption = _('Editing hospital stays ...'),
+		msg = _("The patient's hospitalizations:\n"),
+		caption = _('Editing hospitalizations ...'),
 		columns = [_('Admission'), _('Discharge'), _('Reason'), _('Hospital')],
 		single_selection = True,
 		edit_callback = edit,
 		new_callback = edit,
 		delete_callback = delete,
-		refresh_callback = refresh
+		refresh_callback = refresh,
+		list_tooltip_callback = get_tooltip
 	)
 
 #----------------------------------------------------------------
@@ -485,7 +523,7 @@ def edit_hospital_stay(parent=None, hospital_stay=None):
 	ea.data = hospital_stay
 	ea.mode = gmTools.coalesce(hospital_stay, 'new', 'edit')
 	dlg = gmEditArea.cGenericEditAreaDlg2(parent = parent, id = -1, edit_area = ea, single_entry = True)
-	dlg.SetTitle(gmTools.coalesce(hospital_stay, _('Adding a hospital stay'), _('Editing a hospital stay')))
+	dlg.SetTitle(gmTools.coalesce(hospital_stay, _('Adding a hospitalization'), _('Editing a hospitalization')))
 	if dlg.ShowModal() == wx.ID_OK:
 		dlg.Destroy()
 		return True
@@ -493,8 +531,7 @@ def edit_hospital_stay(parent=None, hospital_stay=None):
 	return False
 #----------------------------------------------------------------
 class cHospitalStayPhraseWheel(gmPhraseWheel.cPhraseWheel):
-	"""Phrasewheel to allow selection of a hospital stay.
-	"""
+	"""Phrasewheel to allow selection of a hospitalization."""
 	def __init__(self, *args, **kwargs):
 
 		gmPhraseWheel.cPhraseWheel.__init__ (self, *args, **kwargs)
@@ -557,21 +594,36 @@ class cHospitalStayEditAreaPnl(wxgHospitalStayEditAreaPnl.wxgHospitalStayEditAre
 
 		valid = True
 
-		if not self._PRW_admission.is_valid_timestamp(allow_empty = False):
-			valid = False
-			gmDispatcher.send(signal = 'statustext', msg = _('Missing admission data. Cannot save hospital stay.'), beep = True)
-
-		if self._PRW_discharge.is_valid_timestamp(allow_empty = True):
-			if self._PRW_discharge.date is not None:
-				if not self._PRW_discharge.date > self._PRW_admission.date:
-					valid = False
-					self._PRW_discharge.display_as_valid(False)
-					gmDispatcher.send(signal = 'statustext', msg = _('Discharge date must be empty or later than admission. Cannot save hospital stay.'), beep = True)
-
 		if self._PRW_episode.GetValue().strip() == u'':
 			valid = False
 			self._PRW_episode.display_as_valid(False)
-			gmDispatcher.send(signal = 'statustext', msg = _('Must select an episode or enter a name for a new one. Cannot save hospital stay.'), beep = True)
+			gmDispatcher.send(signal = 'statustext', msg = _('Must select an episode or enter a name for a new one. Cannot save hospitalization.'), beep = True)
+			self._PRW_episode.SetFocus()
+
+		if not self._PRW_admission.is_valid_timestamp(allow_empty = False):
+			valid = False
+			gmDispatcher.send(signal = 'statustext', msg = _('Missing admission data. Cannot save hospitalization.'), beep = True)
+			self._PRW_admission.SetFocus()
+
+		if self._PRW_discharge.is_valid_timestamp(allow_empty = True):
+			if self._PRW_discharge.date is not None:
+				adm = self._PRW_admission.date
+				discharge = self._PRW_discharge.date
+				# normalize for comparison
+				discharge = discharge.replace (
+					hour = adm.hour,
+					minute = adm.minute,
+					second = adm.second,
+					microsecond = adm.microsecond
+				)
+				if adm is not None:
+					if discharge == adm:
+						self._PRW_discharge.SetData(discharge + pydt.timedelta(seconds = 1))
+					elif not self._PRW_discharge.date > self._PRW_admission.date:
+						valid = False
+						self._PRW_discharge.display_as_valid(False)
+						gmDispatcher.send(signal = 'statustext', msg = _('Discharge date must be empty or later than admission. Cannot save hospitalization.'), beep = True)
+						self._PRW_discharge.SetFocus()
 
 		return (valid is True)
 	#----------------------------------------------------------------
@@ -603,6 +655,7 @@ class cHospitalStayEditAreaPnl(wxgHospitalStayEditAreaPnl.wxgHospitalStayEditAre
 		self._PRW_episode.SetText(value = u'')
 		self._PRW_admission.SetText(data = gmDateTime.pydt_now_here())
 		self._PRW_discharge.SetText()
+		self._PRW_hospital.SetFocus()
 	#----------------------------------------------------------------
 	def _refresh_from_existing(self):
 		if self.data['hospital'] is not None:
@@ -613,9 +666,12 @@ class cHospitalStayEditAreaPnl(wxgHospitalStayEditAreaPnl.wxgHospitalStayEditAre
 
 		self._PRW_admission.SetText(data = self.data['admission'])
 		self._PRW_discharge.SetText(data = self.data['discharge'])
+
+		self._PRW_hospital.SetFocus()
 	#----------------------------------------------------------------
 	def _refresh_as_new_from_existing(self):
 		print "this was not expected to be used in this edit area"
+
 #================================================================
 # encounter related widgets/functions
 #----------------------------------------------------------------
@@ -642,6 +698,9 @@ def edit_encounter(parent=None, encounter=None):
 	dlg.Destroy()
 	return False
 #----------------------------------------------------------------
+def manage_encounters(**kwargs):
+	return select_encounters(**kwargs)
+
 def select_encounters(parent=None, patient=None, single_selection=True, encounters=None, ignore_OK_button=False):
 
 	if patient is None:
@@ -665,7 +724,7 @@ def select_encounters(parent=None, patient=None, single_selection=True, encounte
 
 		items = [
 			[
-				e['started'].strftime('%x  %H:%M'),
+				gmDateTime.pydt_strftime(e['started'], '%Y %b %d  %H:%M'),
 				e['last_affirmed'].strftime('%H:%M'),
 				e['l10n_type'],
 				gmTools.coalesce(e['reason_for_encounter'], u''),
@@ -674,20 +733,39 @@ def select_encounters(parent=None, patient=None, single_selection=True, encounte
 				e['pk_encounter']
 			] for e in encs
 		]
-
 		lctrl.set_string_items(items = items)
 		lctrl.set_data(data = encs)
+		active_pk = emr.active_encounter['pk_encounter']
+		for idx in range(len(encs)):
+			e = encs[idx]
+			if e['pk_encounter'] == active_pk:
+				lctrl.SetItemTextColour(idx, col=wx.NamedColour('RED'))
 	#--------------------
 	def new():
-		enc = gmEMRStructItems.create_encounter(fk_patient = patient.ID)
+		cfg_db = gmCfg.cCfgSQL()
+		# FIXME: look for MRU/MCU encounter type config here
+		enc_type = cfg_db.get2 (
+			option = u'encounter.default_type',
+			workplace = gmSurgery.gmCurrentPractice().active_workplace,
+			bias = u'user',
+			default = u'in surgery'
+		)
+		enc = gmEMRStructItems.create_encounter(fk_patient = patient.ID, enc_type = enc_type)
 		return edit_encounter(parent = parent, encounter = enc)
 	#--------------------
 	def edit(enc=None):
 		return edit_encounter(parent = parent, encounter = enc)
 	#--------------------
+	def edit_active(enc=None):
+		return edit_encounter(parent = parent, encounter = emr.active_encounter)
+	#--------------------
+	def start_new(enc=None):
+		start_new_encounter(emr = emr)
+		return True
+	#--------------------
 	return gmListWidgets.get_choices_from_list (
 		parent = parent,
-		msg = _('\nBelow find the relevant encounters of the patient.\n'),
+		msg = _("The patient's encounters.\n"),
 		caption = _('Encounters ...'),
 		columns = [_('Started'), _('Ended'), _('Type'), _('Reason for Encounter'), _('Assessment of Encounter'), _('Empty'), '#'],
 		can_return_empty = False,
@@ -695,7 +773,9 @@ def select_encounters(parent=None, patient=None, single_selection=True, encounte
 		refresh_callback = refresh,
 		edit_callback = edit,
 		new_callback = new,
-		ignore_OK_button = ignore_OK_button
+		ignore_OK_button = ignore_OK_button,
+		left_extra_button = (_('Edit active'), _('Edit the active encounter'), edit_active),
+		middle_extra_button = (_('Start new'), _('Start new active encounter for the current patient.'), start_new)
 	)
 #----------------------------------------------------------------
 def ask_for_encounter_continuation(msg=None, caption=None, encounter=None, parent=None):
@@ -771,6 +851,77 @@ def edit_encounter_type(parent=None, encounter_type=None):
 		return True
 	return False
 #----------------------------------------------------------------
+class cEncounterPhraseWheel(gmPhraseWheel.cPhraseWheel):
+
+	def __init__(self, *args, **kwargs):
+		gmPhraseWheel.cPhraseWheel.__init__ (self, *args, **kwargs)
+
+		cmd = u"""
+			SELECT DISTINCT ON (list_label)
+				pk_encounter
+					AS data,
+				to_char(started, 'YYYY Mon DD (HH24:MI)') || ': ' || l10n_type || ' [#' || pk_encounter || ']'
+					AS list_label,
+				to_char(started, 'YYYY Mon DD') || ': ' || l10n_type
+					AS field_label
+			FROM
+				clin.v_pat_encounters
+			WHERE
+				(
+					to_char(started, 'YYYY-MM-DD') %(fragment_condition)s
+						OR
+					l10n_type %(fragment_condition)s
+						OR
+					type %(fragment_condition)s
+				)	%(ctxt_patient)s
+			ORDER BY
+				list_label
+			LIMIT
+				30
+		"""
+		context = {'ctxt_patient': {
+			'where_part': u'AND pk_patient = %(patient)s',
+			'placeholder': u'patient'
+		}}
+
+		self.matcher = gmMatchProvider.cMatchProvider_SQL2(queries = [cmd], context = context)
+		self.matcher._SQL_data2match = u"""
+			SELECT
+				pk_encounter
+					AS data,
+				to_char(started, 'YYYY Mon DD (HH24:MI)') || ': ' || l10n_type
+					AS list_label,
+				to_char(started, 'YYYY Mon DD') || ': ' || l10n_type
+					AS field_label
+			FROM
+				clin.v_pat_encounters
+			WHERE
+				pk_encounter = %(pk)s
+		"""
+		self.matcher.setThresholds(1, 3, 5)
+		#self.matcher.print_queries = True
+		self.selection_only = True
+		# outside code MUST bind this to a patient
+		self.set_context(context = 'patient', val = None)
+	#--------------------------------------------------------
+	def set_from_instance(self, instance):
+		val = u'%s: %s' % (
+			gmDateTime.pydt_strftime(instance['started'], '%Y %b %d'),
+			instance['l10n_type']
+		)
+		self.SetText(value = val, data = instance['pk_encounter'])
+	#------------------------------------------------------------
+	def _get_data_tooltip(self):
+		if self.GetData() is None:
+			return None
+		enc = gmEMRStructItems.cEncounter(aPK_obj = self._data.values()[0]['data'])
+		return enc.format (
+			with_docs = False,
+			with_tests = False,
+			with_vaccinations = False,
+			with_family_history = False
+		)
+#----------------------------------------------------------------
 class cEncounterTypePhraseWheel(gmPhraseWheel.cPhraseWheel):
 	"""Phrasewheel to allow selection of encounter type.
 
@@ -816,6 +967,8 @@ ORDER BY
 		self.selection_only = True
 		self.picklist_delay = 50
 #----------------------------------------------------------------
+from Gnumed.wxGladeWidgets import wxgEncounterTypeEditAreaPnl
+
 class cEncounterTypeEditAreaPnl(wxgEncounterTypeEditAreaPnl.wxgEncounterTypeEditAreaPnl, gmEditArea.cGenericEditAreaMixin):
 
 	def __init__(self, *args, **kwargs):
@@ -988,7 +1141,7 @@ class cEncounterEditAreaPnl(wxgEncounterEditAreaPnl.wxgEncounterEditAreaPnl):
 			self._PRW_start.Refresh()
 			self._PRW_start.SetFocus()
 			return False
-		if not self._PRW_start.is_valid_timestamp():
+		if not self._PRW_start.is_valid_timestamp(empty_is_valid = False):
 			self._PRW_start.SetBackgroundColour('pink')
 			self._PRW_start.Refresh()
 			self._PRW_start.SetFocus()
@@ -1068,6 +1221,118 @@ class cEncounterEditAreaDlg(wxgEncounterEditAreaDlg.wxgEncounterEditAreaDlg):
 				self.EndModal(wx.ID_OK)
 			else:
 				self.Close()
+	#--------------------------------------------------------
+	def _on_encounter_start_lost_focus(self):
+		start = self._PRW_encounter_start.GetData()
+		if start is None:
+			return
+		start = start.get_pydt()
+
+		end = self._PRW_encounter_end.GetData()
+		if end is None:
+			fts = gmDateTime.cFuzzyTimestamp (
+				timestamp = start,
+				accuracy = gmDateTime.acc_minutes
+			)
+			self._PRW_encounter_end.SetText(fts.format_accurately(), data = fts)
+			return
+		end = end.get_pydt()
+
+		if start > end:
+			end = end.replace (
+				year = start.year,
+				month = start.month,
+				day = start.day
+			)
+			fts = gmDateTime.cFuzzyTimestamp (
+				timestamp = end,
+				accuracy = gmDateTime.acc_minutes
+			)
+			self._PRW_encounter_end.SetText(fts.format_accurately(), data = fts)
+			return
+
+		emr = self.__pat.get_emr()
+		if start != emr.active_encounter['started']:
+			end = end.replace (
+				year = start.year,
+				month = start.month,
+				day = start.day
+			)
+			fts = gmDateTime.cFuzzyTimestamp (
+				timestamp = end,
+				accuracy = gmDateTime.acc_minutes
+			)
+			self._PRW_encounter_end.SetText(fts.format_accurately(), data = fts)
+			return
+
+		return
+
+#----------------------------------------------------------------
+from Gnumed.wxGladeWidgets import wxgActiveEncounterPnl
+
+class cActiveEncounterPnl(wxgActiveEncounterPnl.wxgActiveEncounterPnl):
+
+	def __init__(self, *args, **kwargs):
+		wxgActiveEncounterPnl.wxgActiveEncounterPnl.__init__(self, *args, **kwargs)
+		self.__register_events()
+		self.refresh()
+	#------------------------------------------------------------
+	def clear(self):
+		self._TCTRL_encounter.SetValue(u'')
+		self._TCTRL_encounter.SetToolTipString(u'')
+		self._BTN_new.Enable(False)
+		self._BTN_list.Enable(False)
+	#------------------------------------------------------------
+	def refresh(self):
+		pat = gmPerson.gmCurrentPatient()
+		if not pat.connected:
+			self.clear()
+			return
+
+		enc = pat.get_emr().active_encounter
+		self._TCTRL_encounter.SetValue(enc.format(with_docs = False, with_tests = False, fancy_header = False, with_vaccinations = False, with_family_history = False).strip('\n'))
+		self._TCTRL_encounter.SetToolTipString (
+			_('The active encounter of the current patient:\n\n%s') %
+				enc.format(with_docs = False, with_tests = False, fancy_header = True, with_vaccinations = False, with_rfe_aoe = True, with_family_history = False).strip('\n')
+		)
+		self._BTN_new.Enable(True)
+		self._BTN_list.Enable(True)
+	#------------------------------------------------------------
+	def __register_events(self):
+		self._TCTRL_encounter.Bind(wx.EVT_LEFT_DCLICK, self._on_ldclick)
+
+		gmDispatcher.connect(signal = u'pre_patient_selection', receiver = self._schedule_clear)
+		# this would throw an exception due to concurrency issues:
+		#gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._schedule_refresh)
+		gmDispatcher.connect(signal = u'episode_mod_db', receiver = self._schedule_refresh)
+		gmDispatcher.connect(signal = u'current_encounter_modified', receiver = self._schedule_refresh)
+		gmDispatcher.connect(signal = u'current_encounter_switched', receiver = self._schedule_refresh)
+	#------------------------------------------------------------
+	# event handler
+	#------------------------------------------------------------
+	def _schedule_clear(self):
+		wx.CallAfter(self.clear)
+	#------------------------------------------------------------
+	def _schedule_refresh(self, *args, **kwargs):
+		wx.CallAfter(self.refresh)
+		return True
+	#------------------------------------------------------------
+	def _on_ldclick(self, event):
+		pat = gmPerson.gmCurrentPatient()
+		if not pat.connected:
+			return
+		edit_encounter(encounter = pat.get_emr().active_encounter)
+	#------------------------------------------------------------
+	def _on_new_button_pressed(self, event):
+		pat = gmPerson.gmCurrentPatient()
+		if not pat.connected:
+			return
+		start_new_encounter(emr = pat.get_emr())
+	#------------------------------------------------------------
+	def _on_list_button_pressed(self, event):
+		if not gmPerson.gmCurrentPatient().connected:
+			return
+		select_encounters()
 #================================================================
 # episode related widgets/functions
 #----------------------------------------------------------------
@@ -1609,6 +1874,37 @@ def edit_health_issue(parent=None, issue=None):
 		return True
 	return False
 #----------------------------------------------------------------
+def select_health_issues(parent=None, emr=None):
+
+	if parent is None:
+		parent = wx.GetApp().GetTopWindow()
+	#-----------------------------------------
+	def refresh(lctrl):
+		issues = emr.get_health_issues()
+		items = [
+			[
+				gmTools.bool2subst(i['is_confidential'], _('CONFIDENTIAL'), u'', u''),
+				i['description'],
+				gmTools.bool2subst(i['clinically_relevant'], _('relevant'), u'', u''),
+				gmTools.bool2subst(i['is_active'], _('active'), u'', u''),
+				gmTools.bool2subst(i['is_cause_of_death'], _('fatal'), u'', u'')
+			] for i in issues
+		]
+		lctrl.set_string_items(items = items)
+		lctrl.set_data(data = issues)
+	#-----------------------------------------
+	return gmListWidgets.get_choices_from_list (
+		parent = parent,
+		msg = _('\nSelect the health issues !\n'),
+		caption = _('Showing health issues ...'),
+		columns = [u'', _('Health issue'), u'', u'', u''],
+		single_selection = False,
+		#edit_callback = edit,
+		#new_callback = edit,
+		#delete_callback = delete,
+		refresh_callback = refresh
+	)
+#----------------------------------------------------------------
 class cIssueListSelectorDlg(gmListWidgets.cGenericListSelectorDlg):
 
 	# FIXME: support pre-selection
@@ -1773,6 +2069,8 @@ ORDER BY
 			self.set_context('pat', patient.ID)
 		return True
 #------------------------------------------------------------
+from Gnumed.wxGladeWidgets import wxgIssueSelectionDlg
+
 class cIssueSelectionDlg(wxgIssueSelectionDlg.wxgIssueSelectionDlg):
 
 	def __init__(self, *args, **kwargs):
@@ -1860,8 +2158,8 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 		self._PRW_age_noted.add_callback_on_lose_focus(self._on_leave_age_noted)
 		self._PRW_year_noted.add_callback_on_lose_focus(self._on_leave_year_noted)
 
-		self._PRW_age_noted.add_callback_on_modified(self._on_modified_age_noted)
-		self._PRW_year_noted.add_callback_on_modified(self._on_modified_year_noted)
+#		self._PRW_age_noted.add_callback_on_modified(self._on_modified_age_noted)
+#		self._PRW_year_noted.add_callback_on_modified(self._on_modified_year_noted)
 
 		self._PRW_year_noted.Enable(True)
 
@@ -1962,7 +2260,7 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 		self._TCTRL_status.SetValue(u'')
 		self._PRW_age_noted.SetText()
 		self._PRW_year_noted.SetText()
-		self._ChBOX_active.SetValue(0)
+		self._ChBOX_active.SetValue(1)
 		self._ChBOX_relevant.SetValue(1)
 		self._ChBOX_confidential.SetValue(0)
 		self._ChBOX_caused_death.SetValue(0)
@@ -2149,6 +2447,8 @@ class cDiagnosticCertaintyClassificationPhraseWheel(gmPhraseWheel.cPhraseWheel):
 #----------------------------------------------------------------
 if __name__ == '__main__':
 
+	from Gnumed.business import gmPersonSearch
+
 	#================================================================	
 	class testapp (wx.App):
 			"""
