@@ -1,8 +1,9 @@
-"""GNUmed form/letter handling widgets.
-"""
+"""GNUmed form/letter handling widgets."""
+
 #================================================================
-__version__ = "$Revision: 1.16 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
+__license__ = "GPL v2 or later"
+
 
 import os.path, sys, logging
 
@@ -12,14 +13,21 @@ import wx
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmI18N, gmTools, gmDispatcher, gmPrinting
-from Gnumed.business import gmForms, gmPerson
-from Gnumed.wxpython import gmGuiHelpers, gmListWidgets, gmMacro
-from Gnumed.wxGladeWidgets import wxgFormTemplateEditAreaPnl, wxgFormTemplateEditAreaDlg
+from Gnumed.pycommon import gmI18N
+from Gnumed.pycommon import gmTools
+from Gnumed.pycommon import gmDispatcher
+from Gnumed.pycommon import gmPrinting
+from Gnumed.pycommon import gmDateTime
+
+from Gnumed.business import gmForms
+from Gnumed.business import gmPerson
+
+from Gnumed.wxpython import gmGuiHelpers
+from Gnumed.wxpython import gmListWidgets
+from Gnumed.wxpython import gmMacro
 
 
 _log = logging.getLogger('gm.ui')
-_log.info(__version__)
 
 #============================================================
 # convenience functions
@@ -30,7 +38,7 @@ def print_doc_from_template(parent=None, jobtype=None, keep_a_copy=True, episode
 		parent = wx.GetApp().GetTopWindow()
 
 	# 1) get template
-	template = manage_form_templates(parent = parent, active_only = True, excluded_types = ['gnuplot script', 'visual progress note'])
+	template = manage_form_templates(parent = parent, active_only = True, excluded_types = ['gnuplot script', 'visual progress note', 'invoice'])
 	if template is None:
 		gmDispatcher.send(signal = 'statustext', msg = _('No document template selected.'))
 		return None
@@ -42,7 +50,7 @@ def print_doc_from_template(parent=None, jobtype=None, keep_a_copy=True, episode
 
 	# 2) process template
 	try:
-		doc = template.instantiate()
+		form = template.instantiate()
 	except KeyError:
 		wx.EndBusyCursor()
 		gmGuiHelpers.gm_show_error (
@@ -52,9 +60,9 @@ def print_doc_from_template(parent=None, jobtype=None, keep_a_copy=True, episode
 		return False
 	ph = gmMacro.gmPlaceholderHandler()
 	#ph.debug = True
-	doc.substitute_placeholders(data_source = ph)
-	doc.edit()
-	printable_file = doc.generate_output()
+	form.substitute_placeholders(data_source = ph)
+	form.edit()
+	printable_file = form.generate_output()
 	if printable_file is None:
 		wx.EndBusyCursor()
 		gmGuiHelpers.gm_show_error (
@@ -67,7 +75,7 @@ def print_doc_from_template(parent=None, jobtype=None, keep_a_copy=True, episode
 	if jobtype is None:
 		jobtype = 'generic_document'
 
-	printed = gmPrinting.print_file_by_shellscript(filename = printable_file, jobtype = jobtype)
+	printed = gmPrinting.print_files(filenames = [printable_file], jobtype = jobtype)
 	if not printed:
 		wx.EndBusyCursor()
 		gmGuiHelpers.gm_show_error (
@@ -88,10 +96,12 @@ def print_doc_from_template(parent=None, jobtype=None, keep_a_copy=True, episode
 
 	# 4) keep a copy
 	if keep_a_copy:
-		# tell UI to import the file
+		files2import = []
+		files2import.extend(form.final_output_filenames)
+		files2import.extend(form.re_editable_filenames)
 		gmDispatcher.send (
-			signal = u'import_document_from_file',
-			filename = printable_file,
+			signal = u'import_document_from_files',
+			filenames = files2import,
 			document_type = template['instance_type'],
 			unlock_patient = True
 		)
@@ -153,7 +163,7 @@ def print_doc_from_ooo_template(template=None):
 
 	return True
 #------------------------------------------------------------
-def manage_form_templates(parent=None, template_types=None, active_only=False, excluded_types=None):
+def manage_form_templates(parent=None, template_types=None, active_only=False, excluded_types=None, msg=None):
 
 	if parent is None:
 		parent = wx.GetApp().GetTopWindow()
@@ -161,8 +171,7 @@ def manage_form_templates(parent=None, template_types=None, active_only=False, e
 	#-------------------------
 	def edit(template=None):
 		dlg = cFormTemplateEditAreaDlg(parent, -1, template=template)
-		result = dlg.ShowModal()
-		return (result == wx.ID_OK)
+		return (dlg.ShowModal() == wx.ID_OK)
 	#-------------------------
 	def delete(template):
 		delete = gmGuiHelpers.gm_show_question (
@@ -189,6 +198,7 @@ def manage_form_templates(parent=None, template_types=None, active_only=False, e
 	#-------------------------
 	template = gmListWidgets.get_choices_from_list (
 		parent = parent,
+		msg = msg,
 		caption = _('Select letter or form template.'),
 		columns = [_('Template'), _('Version'), _('Type')],
 		edit_callback = edit,
@@ -264,6 +274,8 @@ def create_new_letter(parent=None):
 
 	doc.show(True)
 #============================================================
+from Gnumed.wxGladeWidgets import wxgFormTemplateEditAreaPnl
+
 class cFormTemplateEditAreaPnl(wxgFormTemplateEditAreaPnl.wxgFormTemplateEditAreaPnl):
 
 	def __init__(self, *args, **kwargs):
@@ -311,7 +323,7 @@ class cFormTemplateEditAreaPnl(wxgFormTemplateEditAreaPnl.wxgFormTemplateEditAre
 			self._CH_engine.SetSelection(gmForms.form_engine_abbrevs.index(self.__template['engine']))
 			self._CHBOX_active.SetValue(self.__template['in_use'])
 
-			self._TCTRL_date_modified.SetValue(self.__template['last_modified'].strftime('%x'))
+			self._TCTRL_date_modified.SetValue(gmDateTime.pydt_strftime(self.__template['last_modified'], '%Y %b %d'))
 			self._TCTRL_modified_by.SetValue(self.__template['modified_by'])
 
 			self._TCTRL_filename.Enable(True)
@@ -458,6 +470,8 @@ class cFormTemplateEditAreaPnl(wxgFormTemplateEditAreaPnl.wxgFormTemplateEditAre
 
 		dlg.Destroy()
 #============================================================
+from Gnumed.wxGladeWidgets import wxgFormTemplateEditAreaDlg
+
 class cFormTemplateEditAreaDlg(wxgFormTemplateEditAreaDlg.wxgFormTemplateEditAreaDlg):
 
 	def __init__(self, *args, **kwargs):
