@@ -180,9 +180,9 @@ class cPhraseWheelBase(wx.TextCtrl):
 		self._on_modified_callbacks = []
 
 		try:
-			kwargs['style'] = kwargs['style'] | wx.TE_PROCESS_TAB
+			kwargs['style'] = kwargs['style'] | wx.TE_PROCESS_TAB | wx.TE_PROCESS_ENTER
 		except KeyError:
-			kwargs['style'] = wx.TE_PROCESS_TAB
+			kwargs['style'] = wx.TE_PROCESS_TAB | wx.TE_PROCESS_ENTER
 		super(cPhraseWheelBase, self).__init__(parent, id, **kwargs)
 
 		self.__my_startup_color = self.GetBackgroundColour()
@@ -213,6 +213,11 @@ class cPhraseWheelBase(wx.TextCtrl):
 		if value is None:
 			value = u''
 
+		if (value == u'') and (data is None):
+			self._data = {}
+			super(cPhraseWheelBase, self).SetValue(value)
+			return
+
 		self.suppress_text_update_smarts = suppress_smarts
 
 		if data is not None:
@@ -239,6 +244,12 @@ class cPhraseWheelBase(wx.TextCtrl):
 
 		return True
 	#--------------------------------------------------------
+	def set_from_instance(self, instance):
+		raise NotImplementedError('[%s]: set_from_instance()' % self.__class__.__name__)
+	#--------------------------------------------------------
+	def set_from_pk(self, pk):
+		raise NotImplementedError('[%s]: set_from_pk()' % self.__class__.__name__)
+	#--------------------------------------------------------
 	def display_as_valid(self, valid=None, partially_invalid=False):
 		if valid is True:
 			self.SetBackgroundColour(self.__my_startup_color)
@@ -291,7 +302,10 @@ class cPhraseWheelBase(wx.TextCtrl):
 		self._on_lose_focus_callbacks.append(callback)
 	#---------------------------------------------------------
 	def add_callback_on_modified(self, callback=None):
-		"""Add a callback for invocation when the content is modified."""
+		"""Add a callback for invocation when the content is modified.
+
+		This callback will NOT be passed any values.
+		"""
 		if not callable(callback):
 			raise ValueError('[add_callback_on_modified]: ignoring callback [%s] - not callable' % callback)
 
@@ -560,7 +574,9 @@ class cPhraseWheelBase(wx.TextCtrl):
 	# tooltip handling
 	#--------------------------------------------------------
 	def _get_data_tooltip(self):
-		# by default do not support dynamic tooltip parts
+		# child classes can override this to provide
+		# per data item dynamic tooltips,
+		# by default do not support dynamic tooltip parts:
 		return None
 	#--------------------------------------------------------
 	def __recalculate_tooltip(self):
@@ -669,9 +685,10 @@ class cPhraseWheelBase(wx.TextCtrl):
 		self._has_focus = True
 		event.Skip()
 
-		self.__non_edit_font = self.GetFont()
-		edit_font = self.GetFont()
-		edit_font.SetPointSize(pointSize = self.__non_edit_font.GetPointSize() + 1)
+		#self.__non_edit_font = self.GetFont()
+		#edit_font = self.GetFont()
+		edit_font = wx.FontFromNativeInfo(self.__non_edit_font.NativeFontInfo)
+		edit_font.SetPointSize(pointSize = edit_font.GetPointSize() + 1)
 		self.SetFont(edit_font)
 		self.Refresh()
 
@@ -688,13 +705,17 @@ class cPhraseWheelBase(wx.TextCtrl):
 		The user has had her say, so don't second guess
 		intentions but do report error conditions.
 		"""
+		event.Skip()
 		self._has_focus = False
-
 		self.__timer.Stop()
 		self._hide_picklist()
+		wx.CallAfter(self.__on_lost_focus)
+		return True
+	#--------------------------------------------------------
+	def __on_lost_focus(self):
 		self.SetSelection(1,1)
 		self.SetFont(self.__non_edit_font)
-		self.Refresh()
+		#self.Refresh()		# already done in .display_as_valid() below
 
 		is_valid = True
 
@@ -713,9 +734,6 @@ class cPhraseWheelBase(wx.TextCtrl):
 		# notify interested parties
 		for callback in self._on_lose_focus_callbacks:
 			callback()
-
-		event.Skip()
-		return True
 	#--------------------------------------------------------
 	def _on_list_item_selected(self, *args, **kwargs):
 		"""Gets called when user selected a list item."""
@@ -1037,6 +1055,10 @@ class cPhraseWheel(cPhraseWheelBase):
 		The whole thing will only work if data is found
 		in the match space anyways.
 		"""
+		if data is None:
+			self._data = {}
+			return True
+
 		# try getting match candidates
 		self._update_candidates_in_picklist(u'*')
 
@@ -1094,6 +1116,9 @@ class cPhraseWheel(cPhraseWheelBase):
 			if candidate['field_label'] == val:
 				self.data = {candidate['field_label']: candidate}
 				self.MarkDirty()
+				# tell listeners about the user's selection
+				for callback in self._on_selection_callbacks:
+					callback(self._data)
 				return True
 
 		# no exact match found
